"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const argv_1 = __importDefault(require("@meteor-it/argv"));
const fs_1 = require("@meteor-it/fs");
const logger_1 = __importDefault(require("@meteor-it/logger"));
const node_1 = __importDefault(require("@meteor-it/logger/receivers/node"));
const utils_1 = require("@meteor-it/utils");
const path_1 = require("path");
const webpack_1 = __importDefault(require("webpack"));
const webpackConfigGenerator_1 = __importDefault(require("./webpackConfigGenerator"));
logger_1.default.addReceiver(new node_1.default(7));
const parser = new argv_1.default('zarbis');
const logger = parser.logger;
logger.log('Zarbis-Build v' + require('../package.json').version);
process.noDeprecation = true;
function parseRegexpString(f) {
    if (!f.startsWith('/')) {
        throw new Error('Incorrect RegExp');
    }
    const last = f.lastIndexOf('/');
    if (last === 0) {
        throw new Error('Incorrect RegExp');
    }
    return new RegExp(f.slice(1, last), f.slice(last + 1));
}
async function discoverTargets(srcDir) {
    logger.ident(`Discovering targets`);
    let files = await fs_1.walkDirArray(srcDir);
    files = files.map(file => file.split(path_1.sep)).filter(file => file[file.length - 1].startsWith('.target-')).map(file => file.join(path_1.sep));
    logger.log(`Found {green}${files.length}{/green} possible targets:`);
    logger.log(`Validating and getting info`);
    const targets = await utils_1.asyncEach(files, async (path) => {
        const split = path.split(path_1.sep);
        const file = split.slice(0, -1).join(path_1.sep);
        const targetName = split[split.length - 1].replace('.target-', '');
        if (!/[a-zA-Z0-9а-яА-Я_]/.test(targetName))
            throw new Error(`Target name contains not allowed characters! (At {green}${path}{/green})`);
        const target = (await fs_1.readFile(path)).toString();
        if (target.length === 0)
            throw new Error(`Target config file is empty! (At {green}${path}{/green})`);
        const dataLines = target.split('\n').filter(e => e.trim() !== '');
        const data = dataLines.shift().split(';');
        if (data.length !== 2)
            throw new Error(`Target config format is wrong! Should be {green}"for;entrypoint"{/green} (At {green}${path}{/green})`);
        if (!(await fs_1.exists(path_1.resolve(file, data[1]))))
            throw new Error(`Entry file isn't exists! (At {green}${path}{/green})`);
        const internals = dataLines
            .filter(f => f.startsWith('internal '))
            .map(f => f.slice('internal '.length))
            .map(parseRegexpString);
        const externals = dataLines
            .filter(f => f.startsWith('external '))
            .map(f => f.slice('external '.length))
            .map(parseRegexpString);
        if ((internals.length !== 0 || externals.length !== 0) && data[0] === 'web') {
            throw new Error('Internals/externals only supported for node targets (In browser every module is internal)');
        }
        const rest = dataLines.filter(f => !f.startsWith('internal ') && !f.startsWith('external '));
        if (rest.length !== 0) {
            throw new Error(`Unknown lines: ${rest.map(e => `"${e}"`).join('\n')}`);
        }
        return {
            env: data[0],
            name: targetName,
            entry: path_1.resolve(file, data[1]),
            internals,
            externals,
        };
    });
    const names = targets.map(target => target.name);
    if (names.some(function (value) {
        return names.indexOf(value) !== names.lastIndexOf(value);
    })) {
        throw new Error(`Duplicate targets!`);
    }
    logger.ident(`Target list`);
    targets.forEach((target, id) => {
        logger.log(`{green}${(id + 1).toString().padStart(files.length.toString().length, ' ')}{/green}. {blue}${target.name}{/blue} for {blue}${target.env}{/blue}: {blue}${target.entry}{/blue}`);
    });
    logger.deent();
    logger.deent();
    return targets;
}
function initPaths() {
    const projectDir = process.cwd();
    const srcDir = path_1.resolve(projectDir, 'src');
    const distDir = path_1.resolve(projectDir, 'dist');
    logger.ident('Resolving paths');
    logger.log(`Project dir is {green}${projectDir}{/green}`);
    logger.log(`Source code dir is {green}${srcDir}{/green}`);
    logger.log(`Output dir is {green}${distDir}{/green}`);
    logger.deent();
    return { projectDir, srcDir, distDir };
}
async function createWebpack(type, preConfigs, projectDir, _srcDir, distDir) {
    const webpackConfigs = [];
    for (let preConfig of preConfigs) {
        webpackConfigs.push(await webpackConfigGenerator_1.default({
            envVars: {},
            explainErrors: true,
            moduleAliases: {},
            env: type,
            for: preConfig.env,
            entryPoints: {
                main: [preConfig.entry]
            },
            internals: preConfig.internals,
            externals: preConfig.externals,
            outTargetName: preConfig.name,
            outputDirPath: path_1.resolve(distDir, type, preConfig.name),
            projectBaseDir: projectDir
        }));
    }
    const compiler = webpack_1.default(webpackConfigs);
    return compiler;
}
async function startWebpack(type, preConfigs, projectDir, srcDir, distDir) {
    const compiler = await createWebpack(type, preConfigs, projectDir, srcDir, distDir);
    if (type === 'production') {
        compiler.run(() => {
            logger.log(`{green}DONE{/green} Finished production build!`);
            process.exit(0);
        });
    }
    else {
        const ignored = ['**/node_modules/**', '/snapshot/**', '**/.git/**', '**/.gtm/**'];
        compiler.watch(process.env.POLL ? { poll: true, ignored } : {
            poll: false,
            aggregateTimeout: 300,
            ignored
        }, () => logger.log(`{green}DONE{/green} Finished development build!`));
    }
}
parser.command('production')
    .callback(async () => {
    const { projectDir, srcDir, distDir } = initPaths();
    const targets = await discoverTargets(srcDir);
    await startWebpack('production', targets, projectDir, srcDir, distDir);
})
    .help('Compile code');
parser.command('development')
    .callback(async () => {
    const { projectDir, srcDir, distDir } = initPaths();
    const targets = await discoverTargets(srcDir);
    // TODO: Watch for updated .target files
    await startWebpack('development', targets, projectDir, srcDir, distDir);
})
    .help('Watch and recompile code');
parser.help = 'Configuration-less buildsystem';
parser.parse();
//# sourceMappingURL=index.js.map